-- FINVSQRTの繰り返し計算部分
-- X(n+1) = (3 - A*X(n)*X(n))*X(n)/2を行う
-- 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FINVSQRT_repeat_1 is

	port(SRC       : in std_logic_vector(31 downto 0);
        EXP       : out std_logic_vector(7 downto 0);
        Zero      : out std_logic;
        EXPisZero : out std_logic;
	     Xhout     : out std_logic_vector(12 downto 0);
        Xlout     : out std_logic_vector(11 downto 0);
        Aout      : out std_logic_vector(22 downto 0);
	     AXout     : out std_logic_vector(29 downto 0));

end FINVSQRT_repeat_1;

architecture STRUCTURE of FINVSQRT_repeat_1 is

component FINVSQRT_table is
	port(M    : in std_logic_vector(7 downto 0);
	     MANT : out std_logic_vector(6 downto 0));
end component;

signal AX : std_logic_vector(49 downto 0);         -- A*X(n)
signal A : std_logic_vector(24 downto 0);
signal S_EXP : std_logic_vector(7 downto 0);
signal Ah : std_logic_vector(12 downto 0);
signal Al : std_logic_vector(11 downto 0);
signal X : std_logic_vector(24 downto 0);
signal Xh : std_logic_vector(12 downto 0);
signal Xl : std_logic_vector(11 downto 0);
signal AXhh : std_logic_vector(25 downto 0);
signal AXhl : std_logic_vector(24 downto 0);
signal AXlh : std_logic_vector(24 downto 0);
signal AXll : std_logic_vector(23 downto 0);
signal M_buf : std_logic_vector(6 downto 0);
signal M_buf1 : std_logic_vector(24 downto 0);

begin -- STRUCTURE

   S_EXP <= SRC(30 downto 23) - 127;
   EXP <= --(others => '0') when SRC(30 downto 23) = 0 else
          (not (S_EXP(7) & S_EXP(7 downto 1))) + SRC(23) + 127 when SRC(22 downto 0) = 0 else
          (not (S_EXP(7) & S_EXP(7 downto 1))) + 127;
          
   EXPisZero <= '1' when SRC(30 downto 23) = 0 else '0';
          
          
--   EXP <= (not (S_EXP(7) & S_EXP(7 downto 1))) + SRC(23) + 127 when SRC(22 downto 0) = 0 else
--          --(not (S_EXP(7) & S_EXP(7 downto 1))) + (not S_EXP(0)) + 127 when SRC(22 downto 0) = 0 else
--          (not (S_EXP(7) & S_EXP(7 downto 1))) + 127;


   A <= '1' & SRC(22 downto 0) & '0';--SMANT0 & SRC(22 downto 0) & '0';
   FINVSQRT_table7_1 : FINVSQRT_table port map (SRC(23 downto 16),M_buf);
   M_buf1 <= '1' & M_buf & "00000000000000000";

   X <= M_buf1;
   Ah <= A(24 downto 12);
   Al <= A(11 downto 0);
   Xh <= X(24 downto 12);
   Xl <= X(11 downto 0);
   AXhh <= Ah * Xh;
   AXhl <= Ah * Xl;
   AXlh <= Al * Xh;
   AXll <= Al * Xl;
   AX <= (AXhh & AXll) + ("000000000000" & (('0' & AXhl) + ('0' & AXlh))  & "000000000000");

   AXout <= AX(49 downto 20);
   Xhout <= Xh;
   Xlout <= Xl;
   Aout <= SRC(22 downto 0);
   Zero <= '1' when SRC(22 downto 0) = 0 and SRC(23) = '1' else '0';

end STRUCTURE;
